%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Plotting of the two dimensional fluid flow                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Velocities and pressure back to matrices 
                                               
u_new_plot = zeros(M+2,N+1);

u_new_plot(:,1) = u_in_true; 
u_star_plot(:,1) = u_in_true; 

u_new_plot(1,1) = Inf;             % The walls at the inlet are blocked out
u_new_plot(end,1) = Inf; 

for j = 1:M
    for i = 1:N 
        u_new_plot(j+1,i+1) = u_new((j-1)*N + i)*u_in_true;
    end % for
end % for


v_new_plot = zeros(M+1,N+1); 

for j = 1:m                               % The rest of the points are zero
    for i = 1:N 
        v_new_plot(j+1,i+1) = v_new((j-1)*N + i)*u_in_true;
    end % for
end % for                                                 


p_plot = zeros(M,N+1);
p_corrplot = zeros(M,N+1);

p_plot(:,N+1) = p_atm;

for j = 1:M                               % The rest of the points are zero
    for i = 1:N 
        p_plot(j,i) = p_new((j-1)*N + i)*rho_true*u_in_true + p_atm;
%         ;
        p_corrplot(j,i) = p_corr((j-1)*N + i)*rho_true*u_in_true;
    end % for
end % for

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Plot
az_outlet = 45;            % Azimuth angle for setting viewpoint in figures
el_outlet = 30;         % Elevation height for setting viewpoint in figures

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
f1 = figure;
f = surf(xu_plot,yu_plot,u_new_plot);         % surf(x,y,z)
s = sprintf('Plot of $u_{new}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $u$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')
saveas(gcf,'unew2D.png')

f1_outlet = figure;
f = surf(xu_plot,yu_plot,u_new_plot);         % surf(x,y,z)
view(az_outlet, el_outlet)
s = sprintf('Plot of $u_{new}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $u$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')
saveas(gcf,'unewoutlet2D.png')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
f2 = figure;
f = surf(xv_plot,yv_plot,v_new_plot);         % surf(x,y,z)
s = sprintf('Plot of $v_{new}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $v$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')
saveas(gcf,'vnew2D.png')

f2_outlet = figure;
f = surf(xv_plot,yv_plot,v_new_plot);         % surf(x,y,z)
view(az_outlet, el_outlet)
s = sprintf('Plot of $v_{new}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $v$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')
saveas(gcf,'vnewoutlet2D.png')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
f3 = figure;
surf(xp_plot,yp_plot,p_corrplot);         % surf(x,y,z)
s = sprintf('Plot of $p^{corr}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Pressure correction $p''$, [Pa]', 'interpreter', 'latex')
ztickformat('%.2f')
% zlim([-0.1 0.1])
saveas(gcf,'pcorr2D.png')

f3_outlet = figure;
surf(xp_plot,yp_plot,p_corrplot);         % surf(x,y,z)
view(az_outlet, el_outlet)
s = sprintf('Plot of $p^{corr}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Pressure correction $p''$, [Pa]', 'interpreter', 'latex')
% zlim([-0.1 0.1])
ztickformat('%.2f')
saveas(gcf,'pcorroutlet2D.png')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
f4 = figure;
f = surf(xp_plot,yp_plot,p_plot);         % surf(x,y,z)
s = sprintf('Plot of $p_{new}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Pressure $p$, [Pa]', 'interpreter', 'latex')
ztickformat('%.7f')
saveas(gcf,'pnew2D.png')

f4_outlet = figure;
f = surf(xp_plot,yp_plot,p_plot);         % surf(x,y,z)
view(az_outlet, el_outlet)
s = sprintf('Plot of $p_{new}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Pressure $p$, [Pa]', 'interpreter', 'latex')
ztickformat('%.7f')
saveas(gcf,'pnewoutlet2D.png')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Plotting the continuity

 for i = 1:M*N % Global indexing system 
        
        etest = mod(i, N) == 0;
        wtest = mod(i-1, N) == 0;
        ntest = M*N - (N - 1) <= i && i <= N*M  ;
        stest = 1 <= i && i <= N  ;
        
        % Northeastern corner
        if etest == true && ntest == true
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1) ...
                + A_y*v_star(i-N));
            cont_x(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1))  ;
            cont_y(i) = rho*( A_y*v_star(i-N)) ;
         
        % Southeastern corner
        elseif etest == true && stest == true
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1) ...
                -A_y*v_star(i));
            cont_x(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)) ;
            cont_y(i) = rho*(-A_y*v_star(i)) ;

        % Northwestern corner
        elseif wtest == true && ntest == true          
            beta(i) =  rho*(-A_x*u_star(i) +A_x*u_in  ...
                + A_y*v_star(i-N));
            cont_x(i) =  rho*(-A_x*u_star(i) +A_x*u_in);
            cont_y(i) =  rho*(A_y*v_star(i-N));
            
        % Southwestern corner
        elseif wtest == true && stest == true         
            beta(i) =  rho*(-A_x*u_star(i) +A_x*u_in ...
                -A_y*v_star(i));
            cont_x(i) = rho*(-A_x*u_star(i) +A_x*u_in) ;
            cont_y(i) = rho*(-A_y*v_star(i)) ;

        % At eastern boundary (x = L)
        elseif etest == true && ntest == false && stest == false 
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)...
                -A_y*v_star(i) + A_y*v_star(i-N)); 
            cont_x(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)) ;
            cont_y(i) = rho*(-A_y*v_star(i) + A_y*v_star(i-N)) ;
           
        % At western boundary (x = 0)
        elseif wtest == true && ntest == false && stest == false
            beta(i) =  rho*(-A_x*u_star(i) +A_x*u_in ...
                -A_y*v_star(i) + A_y*v_star(i-N));
            cont_x(i) = rho*(-A_x*u_star(i) +A_x*u_in) ;
            cont_y(i) = rho*(-A_y*v_star(i) + A_y*v_star(i-N)) ;
            
        % At northern boundary (y = h)
        elseif ntest == true && etest == false && wtest == false 
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)...
                + A_y*v_star(i-N));
            cont_x(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)) ;
            cont_y(i) = rho*(A_y*v_star(i-N)) ;
            
        % At southern boundary (y = 0) 
        elseif stest == true && etest == false && wtest == false             
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)...
                -A_y*v_star(i));
            cont_x(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1));  
            cont_y(i) = rho*(-A_y*v_star(i));

        %Not at any boundary
        else        
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1) ...
                -A_y*v_star(i) + A_y*v_star(i-N)); 
            cont_x(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1))  ;
            cont_y(i) = rho*(-A_y*v_star(i) + A_y*v_star(i-N)) ;
            
            
        end % if
 end %for 
 
beta_plot = zeros(M,N);
cont_x_plot = zeros(M,N);
cont_y_plot = zeros(M,N);

for j = 1:M                               % the rest of the points are zero
    for i = 1:N 
        cont_x_plot(j,i) = cont_x((j-1)*N + i);
        cont_y_plot(j,i) = cont_y((j-1)*N + i);
        beta_plot(j,i) = beta((j-1)*N + i);
    end % for
end % for


if contplots
    f5 = figure;
    f = surf(xp_plot(1:end-1),yp_plot, cont_x_plot );         % surf(x,y,z)
    s = sprintf(...
        'Plot of $x-$component of continuity after %d iterations', it );
    f = title(s);
    set(f, 'interpreter', 'latex', 'fontsize', 16)
    set(gca,'TickLabelInterpreter','latex')
    xlabel('$x$-direction [m]', 'interpreter', 'latex')
    ylabel('$y$-direction [m]', 'interpreter', 'latex')
    zlabel('Mass flow rate [kg/s]', 'interpreter', 'latex')
    ztickformat('%.2f')
    saveas(gcf,'cont_x.png')


    f6 = figure;
    f = surf(xp_plot(1:end-1),yp_plot, cont_y_plot );         % surf(x,y,z)
    s = sprintf(...
        'Plot of $y-$component of continuity after %d iterations', it );
    f = title(s);
    set(f, 'interpreter', 'latex', 'fontsize', 16)
    set(gca,'TickLabelInterpreter','latex')
    xlabel('$x$-direction [m]', 'interpreter', 'latex')
    ylabel('$y$-direction [m]', 'interpreter', 'latex')
    zlabel('Mass flow rate [kg/s]', 'interpreter', 'latex')
    ztickformat('%.2f')
    saveas(gcf,'cont_y.png')

    f7 = figure;
    f = surf(xp_plot(1:end-1),yp_plot, beta_plot );         % surf(x,y,z)
    s = sprintf(...
        'Plot of $\\beta$ (continuity) after %d iterations', it );
    f = title(s);
    set(f, 'interpreter', 'latex', 'fontsize', 16)
    set(gca,'TickLabelInterpreter','latex')
    xlabel('$x$-direction [m]', 'interpreter', 'latex')
    ylabel('$y$-direction [m]', 'interpreter', 'latex')
    zlabel('Mass flow rate [kg/s]', 'interpreter', 'latex')
    ztickformat('%.2f')
    saveas(gcf,'beta.png')
end