%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Pressure correction script for the BFS model               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

T = zeros(totalpoints, totalpoints); % Initialisation of coefficient matrix
beta = zeros(1, totalpoints);        % Initialisation of source term vector

au = diag(U);           % a^center-coefficients from the momentum equations
av = diag(V);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Calculation
for i = 1:totalpoints                              % Global indexing system 

    etest = ( i <= N_wide*M_wide && mod(i, N_wide) == 0 ) ...  % below step
        || ( i > N_wide*M_wide && mod(i-N_wide*M_wide, N_total) == 0);
    ntest = totalpoints - N_total < i && i <= totalpoints  ;
    wtest = i > N_wide*M_wide && mod(i-1-N_wide*M_wide, N_total) == 0;  
    wwall = i <= N_wide*M_wide  && mod(i-1, N_wide) == 0; 
    stest = (1 <= i && i <= N_wide) ...    % Excluding the corner value 
            || (N_wide*M_wide < i && i <= N_wide*M_wide + N_narrow) ;  
    


    % Northeastern corner
    if etest && ~wtest && ntest && ~stest && ~wwall 

        beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1) ...
            + A_y*v_star(getRowUnder(i, N_wide, M_wide, N_total)));

        % At eastern boundary (x = L)
        eP_coeff = rho*A_x^2/au(i);  

        % At northern boundary (y = h) (y = H)
        nP_coeff = 0 ;  

        W_coeff =  -rho*A_x^2/au(i-1);
        wP_coeff = -W_coeff;
        T(i, i-1) = W_coeff;

        S_coeff = -rho*A_y^2/av(getRowUnder(i, N_wide, M_wide, N_total));
        sP_coeff =  -S_coeff;
        T(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;     

        
    % Southeastern corner
    elseif etest && ~wtest && ~ntest && stest && ~wwall 
        
        beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1) ...
            -A_y*v_star(i));

        % At eastern boundary (x = L)
        eP_coeff = rho*A_x^2/au(i);  

        % At southern boundary (y = 0)
        sP_coeff = 0; 

        W_coeff =  -rho*A_x^2/au(i-1);
        wP_coeff = -W_coeff;
        T(i, i-1) = W_coeff;

        N_coeff = -rho*A_y^2/av(i);
        nP_coeff = -N_coeff;
        T(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        
    % Northwestern corner
    elseif ~etest && wtest && ntest && ~stest && ~wwall 
        
        beta(i) =  rho*(-A_x*u_star(i) ...
            +A_x*u_in(getRowNumber(i, N_wide, M_wide, N_total))  ...
            + A_y*v_star(getRowUnder(i, N_wide, M_wide, N_total)));

        % At western boundary (x = 0)
        wP_coeff = 0;

        % At northern boundary (y = h) (y = H)
        nP_coeff = 0 ;  

        E_coeff = -rho*A_x^2/au(i);
        eP_coeff = -E_coeff ;
        T(i, i+1) = E_coeff;

        S_coeff = -rho*A_y^2/av(getRowUnder(i, N_wide, M_wide, N_total));
        sP_coeff =  -S_coeff;
        T(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;     

        
    % Southwestern corner at inlet 
    elseif ~etest && wtest && ~ntest && stest && ~wwall 
        
        beta(i) =  rho*(-A_x*u_star(i) ...
            +A_x*u_in(getRowNumber(i, N_wide, M_wide, N_total)) ...
            -A_y*v_star(i));

        % At western boundary (x = 0)
        wP_coeff = 0;

        % At southern boundary (y = 0)
        sP_coeff = 0; 

        E_coeff = -rho*A_x^2/au(i);
        eP_coeff = -E_coeff ;
        T(i, i+1) = E_coeff;

        N_coeff = -rho*A_y^2/av(i);
        nP_coeff = -N_coeff;
        T(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

    
    % Southwestern corner at step
    elseif  ~etest && ~wtest && ~ntest && stest && wwall    
        
        beta(i) =  rho*(-A_x*u_circ(i)...
                    +A_x*0 -A_y*v_circ(i)); % wall/"inlet" velocity is zero

        % At western boundary (x = 0)
        wP_coeff = 0;

        % At southern boundary (y = 0)
        sP_coeff = 0; 

        E_coeff = -rho*A_x^2/au(i);
        eP_coeff = -E_coeff ;
        T(i, i+1) = E_coeff;

        N_coeff = -rho*A_y^2/av(i);
        nP_coeff = -N_coeff;
        T(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;


    % At eastern boundary (x = L)
    elseif etest && ~wtest && ~ntest && ~stest && ~wwall 
        
        beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)...
            -A_y*v_star(i) + ...
            A_y*v_star(getRowUnder(i, N_wide, M_wide, N_total))); 

        % At eastern boundary (x = L)
        eP_coeff = rho*A_x^2/au(i);  


        W_coeff =  -rho*A_x^2/au(i-1);
        wP_coeff = -W_coeff;
        T(i, i-1) = W_coeff;

        N_coeff = -rho*A_y^2/av(i);
        nP_coeff = -N_coeff;
        T(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -rho*A_y^2/av(getRowUnder(i, N_wide, M_wide, N_total));
        sP_coeff =  -S_coeff;
        T(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;    

        
    % At western boundary at inlet (x = 0)
    elseif ~etest && wtest && ~ntest && ~stest && ~wwall 
        
        beta(i) =  rho*(-A_x*u_star(i) ...
            +A_x*u_in(getRowNumber(i, N_wide, M_wide, N_total)) ...
            -A_y*v_star(i) ...
            + A_y*v_star(getRowUnder(i, N_wide, M_wide, N_total)));    

        % At western boundary (x = 0)
        wP_coeff = 0;

        E_coeff = -rho*A_x^2/au(i);
        eP_coeff = -E_coeff ;
        T(i, i+1) = E_coeff;

        N_coeff = -rho*A_y^2/av(i);
        nP_coeff = -N_coeff;
        T(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff =- rho*A_y^2/av(getRowUnder(i, N_wide, M_wide, N_total));
        sP_coeff =  -S_coeff;
        T(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;   

    
    % At western wall 
    elseif ~etest && ~wtest && ~ntest && ~stest && wwall 

        beta(i) =  rho*(-A_x*u_circ(i)...  
                    +A_x*0 -A_y*v_circ(i) +... 
                    A_y*v_circ(getRowUnder(i, N_wide, M_wide, N_total)));

        % At western boundary (x = 0)
        wP_coeff = 0;

        E_coeff = -rho*A_x^2/au(i);
        eP_coeff = -E_coeff ;
        T(i, i+1) = E_coeff;

        N_coeff = -rho*A_y^2/av(i);
        nP_coeff = -N_coeff;
        T(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff =- rho*A_y^2/av(getRowUnder(i, N_wide, M_wide, N_total));
        sP_coeff =  -S_coeff;
        T(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;   

        
    % At northern boundary (y = h)
    elseif ~etest && ~wtest && ntest && ~stest && ~wwall  
        
        beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)...
            + A_y*v_star(getRowUnder(i, N_wide, M_wide, N_total))); 

        % At northern boundary (y = h) 
        nP_coeff = 0 ;  

        E_coeff = -rho*A_x^2/au(i);
        eP_coeff = -E_coeff ;
        T(i, i+1) = E_coeff;

        W_coeff =  -rho*A_x^2/au(i-1);
        wP_coeff = -W_coeff;
        T(i, i-1) = W_coeff;

        S_coeff = -rho*A_y^2/av(getRowUnder(i, N_wide, M_wide, N_total));
        sP_coeff =  -S_coeff;
        T(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;     

        
    % At southern boundary (y = 0) 
    elseif ~etest && ~wtest && ~ntest && stest && ~wwall   
        
        beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)...
            -A_y*v_star(i));

        % At southern boundary (y = 0)
        sP_coeff = 0; 

        E_coeff = -rho*A_x^2/au(i);
        eP_coeff = -E_coeff ;
        T(i, i+1) = E_coeff;

        W_coeff =  -rho*A_x^2/au(i-1);
        wP_coeff = -W_coeff;
        T(i, i-1) = W_coeff;

        N_coeff = -rho*A_y^2/av(i);
        nP_coeff = -N_coeff;
        T(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        
    %Not at any boundary
    else 

        beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1) -A_y*v_star(i) + ...
            A_y*v_star(getRowUnder(i, N_wide, M_wide, N_total)));

        E_coeff = -rho*A_x^2/au(i);
        eP_coeff = -E_coeff ;
        T(i, i+1) = E_coeff;

        W_coeff =  -rho*A_x^2/au(i-1);
        wP_coeff = -W_coeff;
        T(i, i-1) = W_coeff;

        N_coeff = -rho*A_y^2/av(i);
        nP_coeff = -N_coeff;
        T(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -rho*A_y^2/av(getRowUnder(i, N_wide, M_wide, N_total));
        sP_coeff =  -S_coeff;
        T(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;                

    end % if

    % Filling in the rest of the matrix, adding all point coefficients
    T(i,i) = wP_coeff + eP_coeff + nP_coeff + sP_coeff;


    etest = false;
    wtest = false;
    ntest = false;
    stest = false;
    wwall = false;

end %for
p_corr = T\beta';                                        % Matrix inversion

        
