%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   u-velocity script for the BFS model                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

U = zeros(totalpoints, totalpoints); % Initialisation of coefficient matrix
bu = zeros(1, totalpoints);          % Initialisation of source term vector

F_xe = zeros(1, totalpoints);    % Initialisation of convective mass fluxes
F_xw = zeros(1, totalpoints);
F_xn = zeros(1, totalpoints);
F_xs = zeros(1, totalpoints);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Generation of F_x, Convective mass fluxes

   
for i = 1:totalpoints

    etest = ( i <= N_wide*M_wide && mod(i, N_wide) == 0 )...   % below step
        || ( i > N_wide*M_wide && mod(i-N_wide*M_wide, N_total) == 0);
    wtest = i > N_wide*M_wide && mod(i-1-N_wide*M_wide, N_total) == 0;  
    ntest = totalpoints - N_total < i && i <= totalpoints  ;
    wwall = i <= N_wide*M_wide  && mod(i-1, N_wide) == 0; 
    stest = (1 <= i && i <= N_wide) ...    % Excluding the corner value 
        || (N_wide*M_wide < i && i < N_wide*M_wide + N_narrow) ;  
    scorner = i == N_wide*M_wide + N_narrow;    % Only the corner value


    % Northeastern corner
    if etest && ~wtest && ntest && ~stest && ~wwall && ~scorner
        F_xe(i) = rho/2*(u_circ(i)+ u_circ(i-1));
        F_xn(i) = 0;                             

        F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
        F_xs(i) = rho/2*v_circ(i-N_total);       

    % Southeastern corner
    elseif etest && ~wtest && ~ntest && stest && ~wwall && ~scorner
        F_xe(i) = rho/2*(u_circ(i)+u_circ(i-1));   
        F_xs(i) = 0;                                

        F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
        F_xn(i) = rho/2*v_circ(i);                  

    % Northwestern corner
    elseif ~etest && wtest && ntest && ~stest && ~wwall && ~scorner
        F_xw(i) = rho/2*(...
            u_in(getRowNumber(i, N_wide, M_wide, N_total))+u_circ(i));           
        F_xn(i) = 0;                                

        F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
        F_xs(i) = rho/2*(v_circ(i-N_total) + v_circ(i-N_total+1));     

    % Southwestern corner at inlet
    elseif ~etest && wtest && ~ntest && stest && ~wwall && ~scorner
        F_xw(i) = rho/2*(...
            u_in(getRowNumber(i, N_wide, M_wide, N_total))+u_circ(i));          
        F_xs(i) = 0;                              

        F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
        F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));

    % Southwestern corner at step
    elseif ~etest && ~wtest && ~ntest && stest && wwall && ~scorner   
        F_xw(i) = rho/2*(0 + u_circ(i));   
        F_xs(i) = 0;                              

        F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
        F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));

    % At corner  
    elseif ~etest && ~wtest && ~ntest && ~stest && ~wwall && scorner 
        F_xs(i) = rho/2*(0 + ... 
            v_circ(getRowUnder(i, N_wide, M_wide, N_total)+1)); 
        F_xs(i)= 0; 

        F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
        F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
        F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));

    % At eastern boundary (x = L)
    elseif etest && ~wtest && ~ntest && ~stest && ~wwall && ~scorner
        F_xe(i) = rho/2*(u_circ(i-1)+u_circ(i));

        F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
        F_xn(i) = rho/2*v_circ(i);
        F_xs(i) = rho/2*v_circ(getRowUnder(i, N_wide, M_wide, N_total));			

    % At western boundary (x = 0)
    elseif ~etest && wtest && ~ntest && ~stest && ~wwall && ~scorner
        F_xw(i) = rho/2*(...
            u_in(getRowNumber(i, N_wide, M_wide, N_total))+u_circ(i));    

        F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
        F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));
        F_xs(i) = rho/2*(...
            v_circ( getRowUnder(i, N_wide, M_wide, N_total)   ) +...
            v_circ( getRowUnder(i, N_wide, M_wide, N_total)+1 )  );

     % At western wall at step 
    elseif ~etest && ~wtest && ~ntest && ~stest && wwall && ~scorner    
        F_xw(i) = rho/2*(0+u_circ(i));    

        F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
        F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));
        F_xs(i) = rho/2*(...
            v_circ( getRowUnder(i, N_wide, M_wide, N_total)   ) +...
            v_circ( getRowUnder(i, N_wide, M_wide, N_total)+1 )  );



    % At northern boundary (y = h)
    elseif ~etest && ~wtest && ntest && ~stest && ~wwall && ~scorner
       F_xn(i) = 0;                              

       F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
       F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
       F_xs(i) = rho/2*(...
            v_circ( getRowUnder(i, N_wide, M_wide, N_total)   ) +...
            v_circ( getRowUnder(i, N_wide, M_wide, N_total)+1 )  );


    % At southern boundary (y = 0) 
    elseif ~etest && ~wtest && ~ntest && stest && ~wwall && ~scorner
        F_xs(i) = 0;                               

        F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
        F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
        F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));    

    % Not at any boundary
    else
        F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
        F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
        F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));
        F_xs(i) = rho/2*(...
            v_circ( getRowUnder(i, N_wide, M_wide, N_total)   ) +...
            v_circ( getRowUnder(i, N_wide, M_wide, N_total)+1 )  );


    end % if
    etest = false;
    wtest = false;
    wwall = false;
    ntest = false;
    stest = false;
    scorner = false;
end %for



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% u-velocity 
    
for i = 1:totalpoints                              % Global indexing system 

    etest = ( i <= N_wide*M_wide && mod(i, N_wide) == 0 )...   % below step
        || ( i > N_wide*M_wide && mod(i-N_wide*M_wide, N_total) == 0);
    wtest = i > N_wide*M_wide && mod(i-1-N_wide*M_wide, N_total) == 0;  
    ntest = totalpoints - N_total < i && i <= totalpoints  ;
    wwall = i <= N_wide*M_wide  && mod(i-1, N_wide) == 0; 
    stest = (1 <= i && i <= N_wide) ...    % Excluding the corner value 
        || (N_wide*M_wide < i && i < N_wide*M_wide + N_narrow) ;  
    scorner = i == N_wide*M_wide + N_narrow;    % Only the corner value
  


    % Northeastern corner
    if etest && ~wtest && ntest && ~stest && ~wwall && ~scorner 
        
        bu(i) =  -(p_out(end)-p_circ(i))*A_x; 
        
        % At eastern boundary (x = L) 
        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff =  F_xe(i)*A_x;  

        % At northern boundary 
        nP_coeff = F_xn(i)*A_y + max(0,-F_xn(i)*A_y) + 2*D_y*A_y;          

        W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_xw(i)*A_x;          
        U(i, i-1) = W_coeff;

        S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_xs(i)*A_y;
        U(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;

        
    % Southeastern corner
    elseif etest && ~wtest && ~ntest && stest && ~wwall && ~scorner 

        bu(i) =  -(p_out(1)-p_circ(i))*A_x; 
        
        % At eastern boundary (x = L) 
        E_coeff = -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff =  F_xe(i)*A_x;    

        % At southern boundary (y = 0)
        sP_coeff = -F_xs(i)*A_y +max(F_xs(i)*A_y,0)+ 2*D_y*A_y;         

        W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_xw(i)*A_x;          
        U(i, i-1) = W_coeff;

        N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_xn(i)*A_y;
        U(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        
    % Northwestern corner
    elseif ~etest && wtest && ntest && ~stest && ~wwall && ~scorner 

        bu(i) = -(p_circ(i+1)-p_circ(i))*A_x +(max(F_xw(i)*A_x,0)...
            + D_x*A_x)*u_in(getRowNumber(i, N_wide, M_wide, N_total)); 
        
        % At western boundary (x = 0) 
        wP_coeff = max(F_xw(i)*A_x,0) + D_x*A_x - F_xw(i)*A_x;

        % At northern boundary 
        nP_coeff = F_xn(i)*A_y + max(0,-F_xn(i)*A_y)+ 2*D_y*A_y;          

        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_xe(i)*A_x;
        U(i, i+1) = E_coeff;

        S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_xs(i)*A_y;
        U(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;          

        
    % Southwestern corner at inlet 
    elseif ~etest && wtest && ~ntest && stest && ~wwall && ~scorner 
     
        bu(i) = -(p_circ(i+1)-p_circ(i))*A_x+(max(F_xw(i)*A_x,0) ...
            + D_x*A_x)*u_in(getRowNumber(i, N_wide, M_wide, N_total)); 
        
        % At western boundary (x = 0) 
        wP_coeff = max(F_xw(i)*A_x,0) + D_x*A_x - F_xw(i)*A_x;

        % At southern boundary (y = 0)
        sP_coeff = -F_xs(i)*A_y +max(F_xs(i)*A_y,0)+ 2*D_y*A_y;         

        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_xe(i)*A_x;
        U(i, i+1) = E_coeff;

        N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_xn(i)*A_y;
        U(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;


    % Southwestern corner at step
    elseif  ~etest && ~wtest && ~ntest && stest && wwall  && ~scorner 

        bu(i) = -(p_circ(i+1)-p_circ(i))*A_x...
            +(max(F_xw(i)*A_x,0) + D_x*A_x)*0; 

        % At western boundary (x = 0) 
        W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_xw(i)*A_x;

        % At southern boundary (y = 0)
        S_coeff = -max(F_xs(i)*A_y,0) - 2*D_y*A_y; 
        sP_coeff = -S_coeff -F_xs(i)*A_y;

        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_xe(i)*A_x;
        U(i, i+1) = E_coeff;

        N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_xn(i)*A_y;
        U(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;


    % At corner 
    elseif ~etest && ~wtest && ~ntest && ~stest && ~wwall && scorner            

        bu(i) = -(p_circ(i+1)-p_circ(i))*A_x;

        % At southern boundary (y = 0)
        S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y; 
        sP_coeff =  -S_coeff - F_xs(i)*A_y;


        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_xe(i)*A_x;
        U(i, i+1) = E_coeff;

        W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_xw(i)*A_x;          
        U(i, i-1) = W_coeff;

        N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_xn(i)*A_y;
        U(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;          


    % At eastern boundary (x = L)
    elseif etest && ~wtest && ~ntest && ~stest && ~wwall && ~scorner 

        bu(i) =  -(p_out(1)-p_circ(i))*A_x;
        
        % At eastern boundary (x = L) 
        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff =  F_xe(i)*A_x;    

        W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_xw(i)*A_x;          
        U(i, i-1) = W_coeff;

        N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_xn(i)*A_y;
        U(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_xs(i)*A_y;
        U(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;          


    % At western boundary (x = 0)
    elseif ~etest && wtest && ~ntest && ~stest && ~wwall && ~scorner 

        bu(i) = -(p_circ(i+1)-p_circ(i))*A_x +(max(F_xw(i)*A_x,0) ...
            + D_x*A_x)*u_in(getRowNumber(i, N_wide, M_wide, N_total)); 
        
        % At western boundary (x = 0) 
        wP_coeff = max(F_xw(i)*A_x,0) + D_x*A_x - F_xw(i)*A_x;

        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_xe(i)*A_x;
        U(i, i+1) = E_coeff;

        N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_xn(i)*A_y;
        U(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_xs(i)*A_y;
        U(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;

        
    % At western wall 
    elseif ~etest && ~wtest && ~ntest && ~stest && wwall && ~scorner 

        bu(i) = -(p_circ(i+1)-p_circ(i))*A_x ...
            +(max(F_xw(i)*A_x,0) + D_x*A_x)*0; 

        % At western boundary (x = 0) 
        W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_xw(i)*A_x;   

        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_xe(i)*A_x;
        U(i, i+1) = E_coeff;

        N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_xn(i)*A_y;
        U(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_xs(i)*A_y;
        U(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;

        
    % At northern boundary (y = h)
    elseif ~etest && ~wtest && ntest && ~stest && ~wwall  && ~scorner 

        bu(i) = -(p_circ(i+1)-p_circ(i))*A_x;
        
        % At northern boundary 
        nP_coeff = F_xn(i)*A_y + max(0,-F_xn(i)*A_y)+ 2*D_y*A_y; 

        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_xe(i)*A_x;
        U(i, i+1) = E_coeff;

        W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_xw(i)*A_x;          
        U(i, i-1) = W_coeff;

        S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_xs(i)*A_y;
        U(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;       

        
    % At southern boundary (y = 0) 
    elseif ~etest && ~wtest && ~ntest && stest && ~wwall  && ~scorner 

        bu(i) = -(p_circ(i+1)-p_circ(i))*A_x;
        
        % At southern boundary (y = 0)
        sP_coeff = -F_xs(i)*A_y +max(F_xs(i)*A_y,0)+ 2*D_y*A_y;        

        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_xe(i)*A_x;
        U(i, i+1) = E_coeff;

        W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_xw(i)*A_x;          
        U(i, i-1) = W_coeff;

        N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_xn(i)*A_y;
        U(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;     


    %Not at any boundary
    else 

        bu(i) = -(p_circ(i+1)-p_circ(i))*A_x;
        E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_xe(i)*A_x;
        U(i, i+1) = E_coeff;

        W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_xw(i)*A_x;          
        U(i, i-1) = W_coeff;

        N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_xn(i)*A_y;
        U(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_xs(i)*A_y;
        U(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;

    end % if

    % Filling in the rest of the matrix, adding all point coefficients
    U(i,i) = wP_coeff + eP_coeff + nP_coeff + sP_coeff;

    etest = false;
    wtest = false;
    ntest = false;
    stest = false;
    wwall = false;

end %for
u_star = U\bu';                                          % Matrix inversion     