%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   v-velocity script for the BFS model                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

V = zeros(totalpoints_v, totalpoints_v);  % Initialisation of coeff. matrix
bv = zeros(1, totalpoints_v);        % Initialisation of source term vector

F_ye = zeros(1, totalpoints_v);  % Initialisation of convective mass fluxes
F_yw = zeros(1, totalpoints_v);
F_yn = zeros(1, totalpoints_v);
F_ys = zeros(1, totalpoints_v);




%% Generation of F_y, Convective mass fluxes

for i = 1:totalpoints_v % Global indexing system 

    % Eastern boundary requires no special treatment (x = L)
    etest = ( i <= N_wide*m_wide && mod(i, N_wide) == 0 ) ...  % below step
          || ( i > N_wide*m_wide && mod(i-N_wide*m_wide, N_total) == 0);
    wtest = i > N_wide*m_wide && mod(i-1-N_wide*m_wide, N_total) == 0;  
    ntest = totalpoints_v - N_total < i && i <= totalpoints_v  ;
    if ~onlyChannel                                           % Normal mode
        wwall = i <= N_wide*m_wide  && mod(i-1, N_wide) == 0; % 
        stest = (1 <= i && i <= N_wide) ...    % Excluding the corner value
                || (N_wide*m_wide < i && i <= N_wide*m_wide + N_narrow) ; 
        wcorner = i == N_wide*(m_wide-1) + 1;       % Only the corner value
    else                                                     % No step mode
        wwall = i <= N_wide*m_wide  && mod(i-1, N_wide) == 0; %  
        stest = i <= N_wide*m_wide + N_total;  % Excluding the corner value
        wcorner = false;                            % Only the corner value
    end %if



    % Northwestern corner
    if  wtest && ntest && ~stest && ~wwall && ~wcorner
        F_yw(i) = rho/2*(u_in+u_in);              
        F_yn(i) = rho/2*v_circ(i);              

        F_ye(i) = rho/2*(u_circ(i) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_ys(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowUnder(i, N_wide, M_wide, N_total)));

    % Southwestern corner at inlet
    elseif  wtest && ~ntest && stest && ~wwall && ~wcorner
        F_yw(i) = rho*u_in;                        
        F_ys(i) = rho/2*v_circ(i); 

        F_ye(i) = rho/2*(u_circ(i) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_yn(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowOver(i, N_wide, M_wide, N_total)));


    % Southwestern corner at step
    elseif  ~wtest && ~ntest && stest && wwall && ~wcorner   
        F_yw(i) = rho*0;                      
        F_ys(i) = rho/2*v_circ(i); 

        F_ye(i) = rho/2*(u_circ(i) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_yn(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowOver(i, N_wide, M_wide, N_total)));


    % At western boundary (x = 0)
    elseif  wtest && ~ntest && ~stest && ~wwall && ~wcorner
        F_yw(i) = rho*u_in;                        

        F_ye(i) = rho/2*(u_circ(i) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_yn(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_ys(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowUnder(i, N_wide, M_wide, N_total)));


    % At western wall 
    elseif  ~wtest && ~ntest && ~stest && wwall && ~wcorner
        F_yw(i) = rho*0;                     

        F_ye(i) = rho/2*(u_circ(i) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_yn(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_ys(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowUnder(i, N_wide, M_wide, N_total)));


    % At corner, right point from the corner
    elseif  ~wtest && ~ntest && ~stest && wwall && wcorner         
        F_yw(i)= 0; 

        F_ye(i) = rho/2*(u_circ(i) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_yn(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_ys(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowUnder(i, N_wide, M_wide, N_total)));

    % At northern boundary (y = h)
    elseif  ~wtest && ntest && ~stest && ~wwall && ~wcorner 
        F_yn(i) = rho/2*v_circ(i);  

        F_ye(i) = rho/2*(u_circ(i) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_yw(i) = rho/2*(u_circ(i-1) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)-1));
        F_ys(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowUnder(i, N_wide, M_wide, N_total)));


    % At southern boundary (y = 0) 
    elseif ~wtest && ~ntest && stest && ~wwall && ~wcorner 
        F_ys(i) = rho/2*v_circ(i);  

        F_ye(i) = rho/2*(u_circ(i) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_yw(i) = rho/2*(u_circ(i-1) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)-1));
        F_yn(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowOver(i, N_wide, M_wide, N_total)));


    %Not at any boundary, including eastern boundary
    else
        F_ye(i) = rho/2*(u_circ(i) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_yw(i) = rho/2*(u_circ(i-1) + ...
            u_circ(getRowOver(i, N_wide, M_wide, N_total)-1));

        F_yn(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowOver(i, N_wide, M_wide, N_total)));
        F_ys(i) = rho/2*(v_circ(i) + ...
            v_circ(getRowUnder(i, N_wide, M_wide, N_total)));

    end % if
    etest = false;
    wtest = false;
    ntest = false;
    stest = false;
    wwall = false;
    wcorner = false; 

end % for
            
%% v-velocity


for i = 1:totalpoints_v                            % Global indexing system 

    etest = ( i <= N_wide*m_wide && mod(i, N_wide) == 0 ) ...  % below step
          || ( i > N_wide*m_wide && mod(i-N_wide*m_wide, N_total) == 0);
    wtest = i > N_wide*m_wide && mod(i-1-N_wide*m_wide, N_total) == 0;  
    ntest = totalpoints_v - N_total < i && i <= totalpoints_v  ;
    if ~onlyChannel                                           % Normal mode
        wwall = i <= N_wide*m_wide  && mod(i-1, N_wide) == 0; % 
        stest = (1 <= i && i <= N_wide) ...    % Excluding the corner value
                || (N_wide*m_wide < i && i <= N_wide*m_wide + N_narrow) ; 
        wcorner = i == N_wide*(m_wide-1) + 1;       % Only the corner value
    else                                                     % No step mode
        wwall = i <= N_wide*m_wide  && mod(i-1, N_wide) == 0; %  
        stest = i <= N_wide*m_wide + N_total;  % Excluding the corner value
        wcorner = false;                            % Only the corner value
    end %if 



    % Northeastern corner
    if etest && ~wtest && ntest && ~stest && ~wwall && ~wcorner

        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y;

        % At eastern boundary (x = L)
        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = F_ye(i)*A_x;

        % At northern boundary 
        nP_coeff = F_yn(i)*A_y + max(0, -F_yn(i)*A_y) + D_y*A_y;  

        W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_yw(i)*A_x;     
        V(i, i-1) = W_coeff;

        S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_ys(i)*A_y;
        V(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;     

    % Southeastern corner
    elseif etest && ~wtest && ~ntest && stest && ~wwall    && ~wcorner
        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y;
        
        % At eastern boundary (x = L)
        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = F_ye(i)*A_x;
 
        % At southern boundary (y = 0), 
        sP_coeff = -F_ys(i)*A_y + max(F_ys(i)*A_y,0) + D_y*A_y; 

        W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_yw(i)*A_x;     
        V(i, i-1) = W_coeff;

        N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_yn(i)*A_y;
        V(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;


    % Northwestern corner
    elseif ~etest && wtest && ntest && ~stest && ~wwall && ~wcorner
        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))-...
                p_circ(i))*A_y + rho*g_y*del_y*A_y;
            
        % At western boundary (x = 0)
        wP_coeff = - F_yw(i)*A_x + max(F_yw(i)*A_x,0) + 2*D_x*A_x;

        % At northern boundary 
        nP_coeff = F_yn(i)*A_y + max(0, -F_yn(i)*A_y) + D_y*A_y ;  

        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_ye(i)*A_x;
        V(i, i+1) = E_coeff;

        S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_ys(i)*A_y;
        V(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;       

    % Southwestern corner at inlet
    elseif ~etest && wtest && ~ntest && stest && ~wwall && ~wcorner

        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y;

        % At western boundary (x = 0)
        wP_coeff = - F_yw(i)*A_x + max(F_yw(i)*A_x,0) + 2*D_x*A_x;

        % At southern boundary (y = 0), 
        sP_coeff = -F_ys(i)*A_y + max(F_ys(i)*A_y,0) + D_y*A_y;  

        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_ye(i)*A_x;
        V(i, i+1) = E_coeff;

        N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_yn(i)*A_y;
        V(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

    % Southwestern corner at step
    elseif ~etest && ~wtest && ~ntest && stest && wwall    && ~wcorner

        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y +...
            0*(-max(F_yw(i)*A_x,0) - 2*D_x*A_x);

        % At western boundary (x = 0)
        W_coeff =  -max(F_yw(i)*A_x,0) - 2*D_x*A_x; 
        wP_coeff = -W_coeff - F_yw(i)*A_x;    

        % At southern boundary (y = 0), 
        S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_ys(i)*A_y;

        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_ye(i)*A_x;
        V(i, i+1) = E_coeff;

        N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_yn(i)*A_y;
        V(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;   

    % At eastern boundary (x = L)
    elseif etest && ~wtest && ~ntest && ~stest && ~wwall && ~wcorner

        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y;

        % At eastern boundary (x = L)
        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = F_ye(i)*A_x;

        W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_yw(i)*A_x;     
        V(i, i-1) = W_coeff;

        N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;
        nP_coeff = -N_coeff + F_yn(i)*A_y;
        V(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_ys(i)*A_y;
        V(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff; 

    % At western boundary (x = 0)
    elseif ~etest && wtest && ~ntest && ~stest && ~wwall && ~wcorner

        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y;

        % At western boundary (x = 0)
        wP_coeff = - F_yw(i)*A_x + max(F_yw(i)*A_x,0) + 2*D_x*A_x;

        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_ye(i)*A_x;
        V(i, i+1) = E_coeff;

        N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_yn(i)*A_y;
        V(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_ys(i)*A_y;
        V(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff; 

    % At west wall (x = 0) [EXCLUDED CORNER] 
    elseif ~etest && ~wtest && ~ntest && ~stest && wwall && ~wcorner

        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y +...
            0*(-max(F_yw(i)*A_x,0) - 2*D_x*A_x); 

        % At western boundary (x = 0)
        W_coeff =  -max(F_yw(i)*A_x,0) - 2*D_x*A_x; 
        wP_coeff = -W_coeff - F_yw(i)*A_x;     

        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_ye(i)*A_x;
        V(i, i+1) = E_coeff;

        N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_yn(i)*A_y;
        V(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_ys(i)*A_y;
        V(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff; 

    % At corner
    elseif ~etest && ~wtest && ~ntest && ~stest && wwall && wcorner
         
        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y +...
            0*(-max(F_yw(i)*A_x,0) - D_x*A_x); 

        % At western boundary (x = 0)
        W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x; 
        wP_coeff = -W_coeff - F_yw(i)*A_x;     

        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_ye(i)*A_x;
        V(i, i+1) = E_coeff;

        N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_yn(i)*A_y;
        V(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_ys(i)*A_y;
        V(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff; 


    % At northern boundary (y = h)
    elseif ~etest && ~wtest && ntest && ~stest && ~wwall && ~wcorner

        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y;

       % At northern boundary 
        nP_coeff = F_yn(i)*A_y + max(0, -F_yn(i)*A_y) + D_y*A_y ;   

        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_ye(i)*A_x;
        V(i, i+1) = E_coeff;

        W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_yw(i)*A_x;     
        V(i, i-1) = W_coeff;

        S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_ys(i)*A_y;
        V(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;      


    % At southern boundary (y = 0) 
    elseif ~etest && ~wtest && ~ntest && stest && ~wwall && ~wcorner

        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y;

        % At southern boundary (y = 0), 
        sP_coeff = -F_ys(i)*A_y + max(F_ys(i)*A_y,0) + D_y*A_y;

        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_ye(i)*A_x;
        V(i, i+1) = E_coeff;

        W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_yw(i)*A_x;     
        V(i, i-1) = W_coeff;

        N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_yn(i)*A_y;
        V(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

    %Not at any boundary
    else 
        
        bv(i) = -(p_circ(getRowOver(i, N_wide, M_wide, N_total))...
            -p_circ(i))*A_y + rho*g_y*del_y*A_y;

        E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
        eP_coeff = -E_coeff + F_ye(i)*A_x;
        V(i, i+1) = E_coeff;

        W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
        wP_coeff = -W_coeff - F_yw(i)*A_x;     
        V(i, i-1) = W_coeff;

        N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
        nP_coeff = -N_coeff + F_yn(i)*A_y;
        V(i, getRowOver(i, N_wide, M_wide, N_total)) = N_coeff;

        S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
        sP_coeff =  -S_coeff - F_ys(i)*A_y;
        V(i, getRowUnder(i, N_wide, M_wide, N_total)) = S_coeff;

    end % if

    % Filling in the rest of the matrix, adding all point coefficients
    V(i,i) = wP_coeff + eP_coeff + nP_coeff + sP_coeff;

    % If the step is disabled the points below the step are blocked out
    if onlyChannel && i <= N_wide*m_wide 
        V(i,i) = V(i,i) + 10e+30;
    end %if

    etest = false;
    wtest = false;
    ntest = false;
    stest = false;
    wwall = false;

end % for
v_star = V\bv';                                          % Matrix inversion
    
    
