%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      Colored velocity quiver plots                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
filler = 0;   % For the quiver plots, the velocities at the step are set to 
              % zero and not Inf, rectangles are therefore used to block
              % out the step from the plots afterwards. 
levels = 50;            % Number of different colors for the representation
showvals = false;                            % Show the value of each color
lines = 'none';                          % Show lines in between each color

% u-velocity
u_fullplot = zeros(M_total+2, N_total+1);
u_fullplot(2:end-1,1) = u_in;
u_fullplot(2:M_wide+1,N_narrow+2:end) = ...
    global2matrix(u_new(1:N_wide*M_wide), N_wide, M_wide);
u_fullplot(M_wide+2:end-1,2:end) = ...
    global2matrix(u_new(N_wide*M_wide+1:end), N_total, M_narrow);
u_fullplot(1:M_wide, 1:N_narrow) = 0;

% Transformation from dimensionless to regular
u_fullplot = u_fullplot*u_in_true; 


% v-velocity
v_fullplot = zeros(m_total+2, N_total+1);
v_fullplot(2:m_wide+1,N_narrow+2:end) = ...
    global2matrix(v_new(1:N_wide*m_wide), N_wide, m_wide);
v_fullplot(m_wide+2:end-1,2:end) = ...
    global2matrix(v_new(N_wide*m_wide+1:end), N_total, m_narrow);
v_fullplot(1:m_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
v_fullplot = v_fullplot*u_in_true; 


uSN = zeros(M_total, N_total);
vSN = zeros(M_total, N_total);
for i = 2:N_total+1
    for j = 1:M_total
        uSN(j,i-1) = 1/2*(u_fullplot(j+1,i-1) + u_fullplot(j+1,i));
    end %for
end %for
for j = 2:M_total+1
    for i = 1:N_total
    	vSN(j-1,i) = 1/2*(v_fullplot(j-1,i) + v_fullplot(j,i));  
    end %for
end %for

% Need to make a combined velocitiy vector
combvel = sqrt(uSN.^2 + vSN.^2);

% Create a mesh for the plotting
[xSN,ySN] = meshgrid(...
    x_0+ del_x_true/2:del_x_true:x_N-del_x_true/2, ...
    y_0+del_y_true/2:del_y_true:y_M-del_y_true/2);

combvelwall = [zeros(1,N_total); combvel ; zeros(1,N_total)];


fq1 = figure;
% Contour plot
[M,c] = contourf([xSN(1,:) ; xSN ;xSN(end,:)],...
    [ones(1,N_total)*y_0; ySN ; ones(1,N_total)*y_M], ...
    combvelwall,levels);
c.LineColor = lines;
hold on
qn = quiver( xSN, ySN , uSN , vSN,'LineWidth',0.5,'Color','k');

%Block out the step    
r = rectangle('Position',[0.03 0 l 1]);
r.FaceColor =  [1 1 1];
r.EdgeColor = 'none';%'k';
r.LineWidth = .0000010;

hold on
set(qn,'AutoScale','on', 'LineWidth',0.1,'AutoScaleFactor', 0.7,...
    'Marker','o','MarkerSize',1,'ShowArrowHead','on')
s = sprintf('Plot of velocities as vectors after %d iterations', it );
% f = title(s);
ax = gca;
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
ax.FontSize = 12;
xlabel('$x$-direction [m]', 'interpreter', 'latex')
xlim([0,L_total])
ylabel('$y$-direction [m]', 'interpreter', 'latex')
ytickformat('%.1f')
set(fq1,'Position', [3   250   717   420]);
saveas(gcf,'velocityquiver.png')
ax.Layer = 'top';


fq2 = figure;
[M,c] = contourf([xSN(1,:) ; xSN ;xSN(end,:)],...
    [ones(1,N_total)*y_0; ySN ; ones(1,N_total)*y_M], ...
    combvelwall,levels);
c.LineColor = lines;
hold on
qn = quiver(...
    xSN, ySN , uSN , vSN,...%u_fullplot(1:end-1,:)
    'LineWidth',0.5,'Color','k');
 
r = rectangle('Position',[0.03 0 l 1]);
r.FaceColor =  [1 1 1];
r.EdgeColor = 'none';%'k';
r.LineWidth = .0000010;

    
hold on
set(qn,'AutoScale','on', 'AutoScaleFactor', 2.1,'Marker','o',...
    'MarkerSize',1,'MaxHeadSize',0.01);%'ShowArrowHead','off')
% qw = quiver(...
%     xv_plot, yv_plot , uplot(1:end-1,:), vplot,...
%         'LineWidth',0.5,'Color','k');
s = sprintf(...
    'Plot of velocities as vectors after %d iterations scales x 1.5', it );
% f = title(s);
ax = gca;
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
ax.FontSize = 12;
xlabel('$x$-direction [m]', 'interpreter', 'latex')
xlim([l-l/4,l*3])
ylabel('$y$-direction [m]', 'interpreter', 'latex')
ylim([0,H+H/4])
ytickformat('%.1f')
set(fq2,'Position', [724   250   560   420]);
saveas(gcf,'velocityquiver1zoomed.png')
ax.Layer = 'top';


fq3 = figure;
[M,c] = contourf([xSN(1,:) ; xSN ;xSN(end,:)],...
    [ones(1,N_total)*y_0; ySN ; ones(1,N_total)*y_M], ...
    combvelwall,levels);
c.LineColor = lines;
hold on
qn = quiver(...
    xSN(1:M_wide,N_narrow+1:N_narrow*2), ...
    ySN(1:M_wide,N_narrow+1:N_narrow*2) ,...
    uSN(1:M_wide,N_narrow+1:N_narrow*2) , ...
    vSN(1:M_wide,N_narrow+1:N_narrow*2),...%u_fullplot(1:end-1,:)
    'LineWidth',0.5,'Color','k');
   
r = rectangle('Position',[0.03 0 l 1]);
r.FaceColor =  [1 1 1];
r.EdgeColor = 'none';%'k';
r.LineWidth = .0000010;

    
hold on
set(qn,'AutoScale','on', 'LineWidth',0.1,'AutoScaleFactor', 2.1,...
    'Marker','o','MarkerSize',1,'ShowArrowHead','on')
% qw = quiver(...
%     xv_plot, yv_plot , uplot(1:end-1,:), vplot,...
%         'LineWidth',0.5,'Color','k');
s = sprintf(...
    'Plot of velocities as vectors after %d iterations, scaled * 2', it );
% f = title(s);
ax = gca;
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
ax.FontSize = 12;
xlabel('$x$-direction [m]', 'interpreter', 'latex')
xlim([l,2*l])
ylabel('$y$-direction [m]', 'interpreter', 'latex')
ylim([0,H])
ytickformat('%.1f')
set(fq3,'Position', [724   250   560   420]);
saveas(gcf,'velocityquiver2zoomed.png')
ax.Layer = 'top';