%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Surface plots for the intermediate velocities              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
showOutletView = true;% Save profiles seen from outlet in addition to inlet 
az = 37.5; % Viewpoints when seen from outlet
el = 30;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Settings
% filler is a value that is filled in where the step is. showStep can be
% adjusted if it is desirable to plot the profiles with zero at the step.
if ~exist('showStep','var') 
    filler = Inf;
else
    if showStep == true
        filler = 0; 
    else
        filler = Inf;
    end %if
end %if

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Velocities to matrices 
% u-velocity

u_fullplot = zeros(M_total+2, N_total+1);
u_fullplot(M_wide+2:end-1,1) = u_in;
u_fullplot(2:M_wide+1,N_narrow+2:end) = ...
    global2matrix(u_star(1:N_wide*M_wide), N_wide, M_wide);
u_fullplot(M_wide+2:end-1,2:end) = ...
    global2matrix(u_star(N_wide*M_wide+1:end), N_total, M_narrow);
u_fullplot(1:M_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
u_fullplot = u_fullplot*u_in_true; 

% Create a mesh for the plotting
[xu_plot,yu_plot] = meshgrid(x_0:del_x_true:x_N,...
    [0, y_0+del_y_true/2:del_y_true:y_M-del_y_true/2, H_total]);

% y-points are adjusted at the inlet because the southern wall of the 
% narrow channel does not align with the u-velocity nodes in the wide sec.
jj = [linspace(0, H_total-h, M_total-M_narrow+1), linspace(H_total-h+del_y_true/2, y_M-del_y_true/2, M_narrow), H_total];
for i = 1:N_narrow 
    for j = 1:M_total
        % alter the points of the plot for the wall of the narrow section
        yu_plot(j,i) = jj(j);
    end %for
end %for

% v-velocity
v_fullplot = zeros(m_total+2, N_total+1);                   % Inlet is zero
v_fullplot(2:m_wide+1,N_narrow+2:end) = ...
    global2matrix(v_star(1:N_wide*m_wide), N_wide, m_wide);
v_fullplot(m_wide+2:end-1,2:end) = ...
    global2matrix(v_star(N_wide*m_wide+1:end), N_total, m_narrow);
v_fullplot(1:m_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
v_fullplot = v_fullplot*u_in_true; 

% Create a mesh for the plotting
[xv_plot,yv_plot] = meshgrid(x_0:del_x_true:x_N, y_0:del_y_true:y_M);


f1 = figure;
f = surf(xu_plot,yu_plot,u_fullplot);         % surf(x,y,z)
% set(f,'edgecolor','none')
s = sprintf('Plot of $u_{star}$ after %d iterations', it );
f = title(s);
set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $u$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')
set(f1, 'Position', [3.6667   40.3333  555.3333  284.6667]); %[left bottom width height]
saveas(gcf,'ustarBFS.png')
if showOutletView
    view(az,el)
    saveas(gcf,'ustaroutletBFS.png')
    view(37.5,30) % back to normal
end % if 

f2 = figure;
f = surf(xv_plot,yv_plot,v_fullplot);         % surf(x,y,z)
% set(f,'edgecolor','none')
s = sprintf('Plot of $v_{star}$ after %d iterations', it );
f = title(s);
set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $v$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')
set(f2, 'Position', [3.6667  327.0000  554.0000  314.0000]); %[left bottom width height]
saveas(gcf,'vstarBFS.png')
if showOutletView
    view(az,el)
    saveas(gcf,'vstaroutletBFS.png')
    view(37.5,30) % back to normal
end % if 

