%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Plots for velocity intermediates and                     %
%                 saved for each specified iteration                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Settings
% filler is a value that is filled in where the step is. showStep can be
% adjusted if it is desirable to plot the profiles with zero at the step.
if ~exist('showStep','var') 
    filler = Inf;
else
    if showStep == true
        filler = 0; 
    else
        filler = Inf;
    end %if
end %if

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% u-velocity
u_circplot = zeros(M_total+2, N_total+1);
u_circplot(M_wide+2:end-1,1) = u_in;
u_circplot(2:M_wide+1,N_narrow+2:end) = ...
    global2matrix(u_circ(1:N_wide*M_wide), N_wide, M_wide);
u_circplot(M_wide+2:end-1,2:end) = ...
    global2matrix(u_circ(N_wide*M_wide+1:end), N_total, M_narrow);
u_circplot(1:M_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
u_circplot = u_circplot*u_in_true; 


u_starplot = zeros(M_total+2, N_total+1);
u_starplot(M_wide+2:end-1,1) = u_in;
u_starplot(2:M_wide+1,N_narrow+2:end) = ...
    global2matrix(u_star(1:N_wide*M_wide), N_wide, M_wide);
u_starplot(M_wide+2:end-1,2:end) = ...
    global2matrix(u_star(N_wide*M_wide+1:end), N_total, M_narrow);
u_starplot(1:M_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
u_starplot = u_starplot*u_in_true; 


u_corrplot = zeros(M_total+2, N_total+1);
u_corrplot(M_wide+2:end-1,1) = 0; % no correction at known
u_corrplot(2:M_wide+1,N_narrow+2:end) = ...
    global2matrix(u_corr(1:N_wide*M_wide), N_wide, M_wide);
u_corrplot(M_wide+2:end-1,2:end) = ...
    global2matrix(u_corr(N_wide*M_wide+1:end), N_total, M_narrow);
u_corrplot(1:M_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
u_corrplot = u_corrplot*u_in_true; 


u_newplot = zeros(M_total+2, N_total+1);
u_newplot(M_wide+2:end-1,1) = u_in;
u_newplot(2:M_wide+1,N_narrow+2:end) = ...
    global2matrix(u_new(1:N_wide*M_wide), N_wide, M_wide);
u_newplot(M_wide+2:end-1,2:end) = ...
    global2matrix(u_new(N_wide*M_wide+1:end), N_total, M_narrow);
u_newplot(1:M_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
u_newplot = u_newplot*u_in_true; 


% Create a mesh for the plotting
[xu_plot,yu_plot] = meshgrid(...
    x_0:del_x_true:x_N, ...
    [0, y_0+del_y_true/2:del_y_true:y_M-del_y_true/2, H_total]);

for i = 1:N_narrow % alter the points of the plot for the wall of the narrow section
    yu_plot(:,i) = [linspace(0, H_total-h, M_total-M_narrow+1), linspace(H_total-h+del_y_true/2, y_M-del_y_true/2, M_narrow), H_total];
%     yu_plot(i,:) = [linspace(0, H_total-h, M-M+1), linspace(H_total-h+del_y_true/2, y_M-del_y_true/2, M), H_total];
end %for


f0 = figure('units','normalized','outerposition',[0 0 1 1]);


subplot(2,2,1);
f = surf(xu_plot,yu_plot,u_circplot);         % surf(x,y,z)

s = sprintf('Plot of $u_{circ}$ after %d iterations', it );
f = title(s);
set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $u$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')


subplot(2,2,3);
f = surf(xu_plot,yu_plot,u_starplot);         % surf(x,y,z)
s = sprintf('Plot of $u_{star}$ after %d iterations', it );
f = title(s);
set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $v$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')


subplot(2,2,2);
f = surf(xu_plot,yu_plot,u_corrplot);         % surf(x,y,z)
s = sprintf('Plot of $u_{corr}$ after %d iterations', it );
f = title(s);
set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')


subplot(2,2,4);
f = surf(xu_plot,yu_plot,u_newplot);         % surf(x,y,z)
s = sprintf('Plot of $u_{new}$ after %d iterations', it );
f = title(s);
set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')

    
    
axis tight manual % this ensures that getframe() returns a consistent size
filename = 'itdev_uintermediates.gif';
% Capture the plot as an image 
frame = getframe(f0); 
im = frame2im(frame); 
[imind,cm] = rgb2ind(im,256); 
% Write to the GIF File 
if it == 1
  imwrite(imind,cm,filename,'gif', 'Loopcount',inf); 
else 
  imwrite(imind,cm,filename,'gif','WriteMode','append'); 
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% v-velocity
v_circplot = zeros(m_total+2, N_total+1);
v_circplot(2:m_wide+1,N_narrow+2:end) = ...
    global2matrix(v_circ(1:N_wide*m_wide), N_wide, m_wide);
v_circplot(m_wide+2:end-1,2:end) = ...
    global2matrix(v_circ(N_wide*m_wide+1:end), N_total, m_narrow);
v_circplot(1:m_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
v_circplot = v_circplot*u_in_true; 


v_starplot = zeros(m_total+2, N_total+1);
v_starplot(2:m_wide+1,N_narrow+2:end) = ...
    global2matrix(v_star(1:N_wide*m_wide), N_wide, m_wide);
v_starplot(m_wide+2:end-1,2:end) = ...
    global2matrix(v_star(N_wide*m_wide+1:end), N_total, m_narrow);
v_starplot(1:m_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
v_starplot = v_starplot*u_in_true; 


v_corrplot = zeros(m_total+2, N_total+1);
v_corrplot(2:m_wide+1,N_narrow+2:end) = ...
    global2matrix(v_corr(1:N_wide*m_wide), N_wide, m_wide);
v_corrplot(m_wide+2:end-1,2:end) = ...
    global2matrix(v_corr(N_wide*m_wide+1:end), N_total, m_narrow);
v_corrplot(1:m_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
v_corrplot = v_corrplot*u_in_true; 


v_newplot = zeros(m_total+2, N_total+1);
v_newplot(2:m_wide+1,N_narrow+2:end) = ...
    global2matrix(v_new(1:N_wide*m_wide), N_wide, m_wide);
v_newplot(m_wide+2:end-1,2:end) = ...
    global2matrix(v_new(N_wide*m_wide+1:end), N_total, m_narrow);
v_newplot(1:m_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
v_newplot = v_newplot*u_in_true; 


% Create a mesh for the plotting
[xv_plot,yv_plot] = meshgrid(...
    x_0:del_x_true:x_N, ...
    y_0:del_y_true:y_M);


f2 = figure('units','normalized','outerposition',[0 0 1 1]);


subplot(2,2,1);
f = surf(xv_plot,yv_plot,v_circplot);         % surf(x,y,z)
s = sprintf('Plot of $v_{circ}$ after %d iterations', it );
f = title(s);
set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $u$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')


subplot(2,2,3);
f = surf(xv_plot,yv_plot,v_starplot);         % surf(x,y,z)
s = sprintf('Plot of $v_{star}$ after %d iterations', it );
f = title(s);
set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')


subplot(2,2,2);
f = surf(xv_plot,yv_plot,v_corrplot);         % surf(x,y,z)
s = sprintf('Plot of $v_{corr}$ after %d iterations', it );
f = title(s);
set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')


subplot(2,2,4);
f = surf(xv_plot,yv_plot,v_newplot);        
s = sprintf('Plot of $v_{new}$ after %d iterations', it );
f = title(s);
set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Pressure $p$, [Pa]', 'interpreter', 'latex')
ztickformat('%.2f')


axis tight manual % this ensures that getframe() returns a consistent size
filename = 'itdev_vintermediates.gif';
% Capture the plot as an image 
frame = getframe(f2); 
im = frame2im(frame); 
[imind,cm] = rgb2ind(im,256); 
% Write to the GIF File 
if it == 1 
  imwrite(imind,cm,filename,'gif', 'Loopcount',inf); 
else 
  imwrite(imind,cm,filename,'gif','WriteMode','append'); 
end 
close all



