%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Surface plots for velocities, pressure and pressure correction      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
showOutletView = true;% Save profiles seen from outlet in addition to inlet 
az = 37.5; % Viewpoints when seen from outlet
el = 30;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Settings
% filler is a value that is filled in where the step is. showStep can be
% adjusted if it is desirable to plot the profiles with zero at the step.
if ~exist('showStep','var') 
    filler = Inf;
else
    if showStep == true
        filler = 0; 
    else
        filler = Inf;
    end %if
end %if

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Velocities to matrices 
% u-velocity

u_fullplot = zeros(M_total+2, N_total+1);
u_fullplot(M_wide+2:end-1,1) = u_in;
u_fullplot(2:M_wide+1,N_narrow+2:end) = ...
    global2matrix(u_new(1:N_wide*M_wide), N_wide, M_wide);
u_fullplot(M_wide+2:end-1,2:end) = ...
    global2matrix(u_new(N_wide*M_wide+1:end), N_total, M_narrow);
u_fullplot(1:M_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
u_fullplot = u_fullplot*u_in_true; 

% Create a mesh for the plotting
[xu_plot,yu_plot] = meshgrid(x_0:del_x_true:x_N,...
    [0, y_0+del_y_true/2:del_y_true:y_M-del_y_true/2, H_total]);

% y-points are adjusted at the inlet because the southern wall of the 
% narrow channel does not align with the u-velocity nodes in the wide sec.
jj = [linspace(0, H_total-h, M_total-M_narrow+1),...
    linspace(H_total-h+del_y_true/2, y_M-del_y_true/2, M_narrow), H_total];
for i = 1:N_narrow 
    for j = 1:M_total
        % alter the points of the plot for the wall of the narrow section
        yu_plot(j,i) = jj(j);
    end %for
end %for

% v-velocity
v_fullplot = zeros(m_total+2, N_total+1);                   % Inlet is zero
v_fullplot(2:m_wide+1,N_narrow+2:end) = ...
    global2matrix(v_new(1:N_wide*m_wide), N_wide, m_wide);
v_fullplot(m_wide+2:end-1,2:end) = ...
    global2matrix(v_new(N_wide*m_wide+1:end), N_total, m_narrow);
v_fullplot(1:m_wide, 1:N_narrow) = filler;

% Transformation from dimensionless to regular
v_fullplot = v_fullplot*u_in_true; 

% Create a mesh for the plotting
[xv_plot,yv_plot] = meshgrid(x_0:del_x_true:x_N, y_0:del_y_true:y_M);


f1 = figure;
f = surf(xu_plot,yu_plot,u_fullplot);         
% set(f,'edgecolor','none')
s = sprintf('Plot of $u_{new}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $u$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')
set(f1, 'Position', [3.6667   40.3333  555.3333  284.6667]); 
%[left bottom width height]
saveas(gcf,'unewBFS.png')
if showOutletView
    view(az,el)
    saveas(gcf,'unewoutletBFS.png')
    view(37.5,30) % back to normal
end % if 


f2 = figure;
f = surf(xv_plot,yv_plot,v_fullplot);         % surf(x,y,z)
% set(f,'edgecolor','none')
s = sprintf('Plot of $v_{new}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Velocity $v$, [m/s]', 'interpreter', 'latex')
ztickformat('%.2f')
set(f2, 'Position', [3.6667  327.0000  554.0000  314.0000]); 
%[left bottom width height]
saveas(gcf,'vnewBFS.png')
if showOutletView
    view(az,el)
    saveas(gcf,'vnewoutletBFS.png')
    view(37.5,30) % back to normal
end % if 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Pressure to matrix

p_fullplot = zeros(M_total, N_total+1);
p_fullplot(1:M_wide,N_narrow+1:end-1) = ...
    global2matrix(p_new(1:N_wide*M_wide), N_wide, M_wide);
p_fullplot(M_wide+1:end,1:end-1) = ...
    global2matrix(p_new(N_wide*M_wide+1:end), N_total, M_narrow);
p_fullplot(1:M_wide, 1:N_narrow) = filler;
p_fullplot(:, end) = p_out;

% Transformation from dimensionless to regular
p_fullplot = p_fullplot*rho_true*u_in_true + p_atm;

% Create a mesh for the plotting
[xp_plot,yp_plot] = meshgrid(...
    x_0:del_x_true:x_N, ...
    y_0+del_y_true/2:del_y_true:y_M-del_y_true/2);
xp_plot = xp_plot + del_x_true/2;
yp_plot = yp_plot + del_y_true/2;

f3 = figure;
f = surf(xp_plot,yp_plot,p_fullplot);       
% set(f,'edgecolor','none')
s = sprintf('Plot of $p_{new}$ after %d iterations', it );
% f = title(s);
% set(f, 'interpreter', 'latex', 'fontsize', 16)
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
zlabel('Pressure $p$, [Pa]', 'interpreter', 'latex')
ztickformat('%.7f')
set(f3, 'Position', [ 721.6667   40.3333  560.0000  287.3333]); 
%[left bottom width height]
saveas(gcf,'pnewBFS.png')
if showOutletView
    view(az,el)
    saveas(gcf,'pnewoutletBFS.png')
    view(37.5,30) % back to normal
end % if 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Pressure correction to matrix
if it > 0 % Don't plot in case of initial profiles (plotinitialprofiles)
    
    p_corrplot = zeros(M_total, N_total+1);
    p_corrplot(1:M_wide,N_narrow+1:end-1) = ...
        global2matrix(p_corr(1:N_wide*M_wide), N_wide, M_wide);
    p_corrplot(M_wide+1:end,1:end-1) = ...
        global2matrix(p_corr(N_wide*M_wide+1:end), N_total, M_narrow);
    p_corrplot(1:M_wide, 1:N_narrow) = filler;
    
    % Transformation from dimensionless to regular
    p_corrplot = p_corrplot*rho_true*u_in_true;
        
    % Create a mesh for the plotting
    [xp_plot,yp_plot] = meshgrid(...
        x_0:del_x_true:x_N, ...
        y_0+del_y_true/2:del_y_true:y_M-del_y_true/2);

    f4 = figure;
    f = surf(xp_plot,yp_plot,p_corrplot);         % surf(x,y,z)
    % set(f,'edgecolor','none')
    s = sprintf('Plot of $p_{corr}$ after %d iterations', it );
%     f = title(s);
%     set(f, 'interpreter', 'latex', 'fontsize', 16)
    set(gca,'TickLabelInterpreter','latex')
    xlabel('$x$-direction [m]', 'interpreter', 'latex')
    ylabel('$y$-direction [m]', 'interpreter', 'latex')
	ztickformat('%.2f')
    zlabel('Pressure correction $p''$, [Pa]', 'interpreter', 'latex')
    set(f4, 'Position', [719.6667  329.6667  560.0000  311.3333]); 
    saveas(gcf,'pcorrBFS.png')
    
    
    if showOutletView
        view(az,el)
        saveas(gcf,'pcorroutletBFS.png')
        view(37.5,30) % back to normal
    end % if 
end %if










