%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                        Elliptic grid generation                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clc
clear

maxits = 100;

P1 = 0;                                          % Poisson control function
P2 = 0;                                          % Poisson control function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Create the algebraic grid for an initial guess
transfinite
N = length(q1);
M = length(q2);
n = N-2;            % dimensions of the inner point matrix to be solved for 
m = M-2;                % with the elliptic grid generation equations below
alpha = 0.001;

conv = 0;
it = 1;

while conv == 0
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Area Components
    AM11 = zeros(m,n); % Indexed top bottom 
    AM12 = zeros(m,n); % A^1_2
    AM21 = zeros(m,n); % A^2_1
    AM22 = zeros(m,n);
    for i = 2:N-1
        for j = 2:M-1
            AM11(j-1,i-1) = 1/2*(y(j+1,i) - y(j-1,i));
            AM21(j-1,i-1) = -1/2*(y(j,i+1) - y(j,i-1));
            AM12(j-1,i-1) = -1/2*(x(j+1,i) - x(j-1,i));
            AM22(j-1,i-1) = 1/2*(x(j,i+1) - x(j,i-1));
        end %for
    end %for

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Jacobi Determinant
    J2 = zeros(m,n);
    for i = 2:N-1
        for j = 2:M-1
            J2(j-1,i-1) = (1/4*(x(j,i+1)-x(j,i-1))*(y(j+1,i)-y(j-1,i))...
                        - 1/4*(y(j,i+1)-y(j,i-1))*(x(j+1,i)-x(j-1,i)))^2;
        end %for
    end %for
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Contravariant Tensor Components
    gM11 = zeros(m,n);
    gM12 = zeros(m,n);
    gM21 = zeros(m,n);
    gM22 = zeros(m,n);
    for i = 1:n
        for j = 1:m
            gM11(j,i) = 1/J2(j,i)*...
                (AM11(j,i)*AM11(j,i) + AM12(j,i)*AM12(j,i));
            gM21(j,i) = 1/J2(j,i)*...
                (AM21(j,i)*AM11(j,i) + AM22(j,i)*AM12(j,i));
            gM12(j,i) = 1/J2(j,i)*...
                (AM11(j,i)*AM21(j,i) + AM12(j,i)*AM22(j,i));
            gM22(j,i) = 1/J2(j,i)*...
                (AM21(j,i)*AM21(j,i) + AM22(j,i)*AM22(j,i));
        end %for
    end %for
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Matrices 2 Globals
    A11 = matrix2global(AM11,n,m);
    A12 = matrix2global(AM12,n,m);
    A21 = matrix2global(AM21,n,m);
    A22 = matrix2global(AM22,n,m);
    
    g11 = matrix2global(gM11,n,m);
    g12 = matrix2global(gM12,n,m);
    g21 = matrix2global(gM21,n,m);
    g22 = matrix2global(gM22,n,m);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% New x and y
    X = zeros(n*m,n*m);
    Y = zeros(n*m,n*m);
    
    % The source term is zero and is updated if the point is at a boundary
    bx = zeros(1,n*m); 
    by = zeros(1,n*m); 
    
    
    for i = 1:n*m
        
        etest = mod(i, n) == 0; 
        ntest = n*m - n < i;
        wtest =  mod(i-1, n) == 0;
        stest = i <= n;

        % Northeastern corner
        if etest && ~wtest && ntest && ~stest
            
            X(i,i) =  -2*g11(i)-2*g22(i);
            % X(i,i+1)=(g11(i)+P1/2) ;
            bx(i) = bx(i) - x(getRow(i,n),getCol(i,n)+1)*(g11(i)+P1/2);
            X(i,i-1)=(g11(i)-P1/2) ;
            % X(i+n,i)=(g22(i)+P2/2) ;
            bx(i) = bx(i) - x(getRow(i,n)+1,getCol(i,n))*(g22(i)+P2/2);
            X(i-n,i)=(g22(i)-P2/2) ;
            % X(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            bx(i) = bx(i)...
                - x(getRow(i,n)+1,getCol(i,n)+1)*(g12(i)/4+g21(i)/4) ;
            %  X(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            bx(i) = bx(i)...
                - x(getRow(i,n)+1,getCol(i,n)-1)*(-g12(i)/4-g21(i)/4) ;
            % X(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            bx(i) = bx(i)...
                - x(getRow(i,n)-1,getCol(i,n)+1)*(-g12(i)/4-g21(i)/4);
            X(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            
            Y(i,i) =  -2*g11(i)-2*g22(i);
            % Y(i,i+1)=(g11(i)+P1/2) ;
            by(i) = by(i) - y(getRow(i,n),getCol(i,n)+1)*(g11(i)+P1/2);
            Y(i,i-1)=(g11(i)-P1/2) ;
            % Y(i+n,i)=(g22(i)+P2/2) ;
            by(i) = by(i) - y(getRow(i,n)+1,getCol(i,n))*(g22(i)+P2/2);
            Y(i-n,i)=(g22(i)-P2/2) ;
            % Y(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            by(i) = by(i)...
                - y(getRow(i,n)+1,getCol(i,n)+1)*(g12(i)/4+g21(i)/4) ;
            % Y(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            by(i) = by(i)...
                - y(getRow(i,n)+1,getCol(i,n)-1)*(-g12(i)/4-g21(i)/4) ;
            % Y(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            by(i) = by(i)...
                - y(getRow(i,n)-1,getCol(i,n)+1)*(-g12(i)/4-g21(i)/4);
            Y(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            
        % Southeastern corner
        elseif etest && ~wtest && ~ntest && stest          
            
            X(i,i) = -2*g11(i)-2*g22(i);
            % X(i,i+1)=(g11(i)+P1/2) ;
            bx(i) = bx(i) - x(getRow(i,n),getCol(i,n)+1)*(g11(i)+P1/2);
            X(i,i-1)=(g11(i)-P1/2) ;
            X(i+n,i)=(g22(i)+P2/2) ;
            % X(i-n,i)=(g22(i)-P2/2) ;
            bx(i) = bx(i) - x(getRow(i,n)-1,getCol(i,n))*(g22(i)-P2/2);
            % X(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            bx(i) = bx(i)...
                - x(getRow(i,n)+1,getCol(i,n)+1)*(g12(i)/4+g21(i)/4);
            X(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            % X(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            bx(i) = bx(i)...
                - x(getRow(i,n)-1,getCol(i,n)+1)*(-g12(i)/4-g21(i)/4);
            % X(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            bx(i) = bx(i)...
                - x(getRow(i,n)-1,getCol(i,n)-1)*(g12(i)/4+g21(i)/4);
            
                        
            Y(i,i) = -2*g11(i)-2*g22(i);
            % Y(i,i+1)=(g11(i)+P1/2) ;
            by(i) = by(i) - y(getRow(i,n),getCol(i,n)+1)*(g11(i)+P1/2);
            Y(i,i-1)=(g11(i)-P1/2) ;
            Y(i+n,i)=(g22(i)+P2/2) ;
            % Y(i-n,i)=(g22(i)-P2/2) ;
            by(i) = by(i) - y(getRow(i,n)-1,getCol(i,n))*(g22(i)-P2/2);
            % Y(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            by(i) = by(i)...
                - y(getRow(i,n)+1,getCol(i,n)+1)*(g12(i)/4+g21(i)/4);
            Y(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            % Y(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            by(i) = by(i)...
                - y(getRow(i,n)-1,getCol(i,n)+1)*(-g12(i)/4-g21(i)/4);
            % Y(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            by(i) = by(i)...
                - y(getRow(i,n)-1,getCol(i,n)-1)*(g12(i)/4+g21(i)/4);

        % Northwestern corner
        elseif ~etest && wtest && ntest && ~stest 
            
            X(i,i) =  -2*g11(i)-2*g22(i);
            X(i,i+1)=(g11(i)+P1/2) ;
            % X(i,i-1)=(g11(i)-P1/2) ;
            bx(i) = bx(i) - x(getRow(i,n),getCol(i,n)-1)*(g11(i)-P1/2);
            % X(i+n,i)=(g22(i)+P2/2) ;
            bx(i) = bx(i) - x(getRow(i,n)+1,getCol(i,n))*(g22(i)+P2/2);
            X(i-n,i)=(g22(i)-P2/2) ;
            % X(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            bx(i) = bx(i)...
                - x(getRow(i,n)+1,getCol(i,n)+1)*(g12(i)/4+g21(i)/4) ;
            % X(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            bx(i) = bx(i)...
                - x(getRow(i,n)+1,getCol(i,n)-1)*(-g12(i)/4-g21(i)/4) ;
            X(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            % X(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            bx(i) = bx(i)...
                - x(getRow(i,n)-1,getCol(i,n)-1)*(g12(i)/4+g21(i)/4);
            
            Y(i,i) =  -2*g11(i)-2*g22(i);
            Y(i,i+1)=(g11(i)+P1/2) ;
            % Y(i,i-1)=(g11(i)-P1/2) ;
            by(i) = by(i) - y(getRow(i,n),getCol(i,n)-1)*(g11(i)-P1/2);
            % Y(i+n,i)=(g22(i)+P2/2) ;
            by(i) = by(i) - y(getRow(i,n)+1,getCol(i,n))*(g22(i)+P2/2);
            Y(i-n,i)=(g22(i)-P2/2) ;
            % Y(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            by(i) = by(i)...
                - y(getRow(i,n)+1,getCol(i,n)+1)*(g12(i)/4+g21(i)/4) ;
            % Y(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            by(i) = by(i)...
                - y(getRow(i,n)+1,getCol(i,n)-1)*(-g12(i)/4-g21(i)/4) ;
            Y(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            % Y(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            by(i) = by(i)...
                - y(getRow(i,n)-1,getCol(i,n)-1)*(g12(i)/4+g21(i)/4);
            

        % Southwestern corner
        elseif ~etest && wtest && ~ntest && stest                       

            X(i,i) = -2*g11(i)-2*g22(i);
            X(i,i+1)=(g11(i)+P1/2) ;
            % X(i,i-1)=(g11(i)-P1/2) ;
            bx(i) = bx(i) - x(getRow(i,n),getCol(i,n)-1)*(g11(i)-P1/2);
            X(i+n,i)=(g22(i)+P2/2) ;
            % X(i-n,i)=(g22(i)-P2/2) ;
            bx(i) = bx(i) - x(getRow(i,n)-1,getCol(i,n))*(g22(i)-P2/2);
            X(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            % X(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            bx(i) = bx(i)...
                - x(getRow(i,n)+1,getCol(i,n)-1)*(-g12(i)/4-g21(i)/4);
            % X(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            bx(i) = bx(i)...
                - x(getRow(i,n)-1,getCol(i,n)+1)*(-g12(i)/4-g21(i)/4);
            % X(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            bx(i) = bx(i)...
                - x(getRow(i,n)-1,getCol(i,n)-1)*(g12(i)/4+g21(i)/4);

            Y(i,i) = -2*g11(i)-2*g22(i);
            Y(i,i+1)=(g11(i)+P1/2) ;
            % Y(i,i-1)=(g11(i)-P1/2) ;
            by(i) = by(i) - y(getRow(i,n),getCol(i,n)-1)*(g11(i)-P1/2);
            Y(i+n,i)=(g22(i)+P2/2) ;
            % Y(i-n,i)=(g22(i)-P2/2) ;
            by(i) = by(i)...
                - y(getRow(i,n)-1,getCol(i,n))*(g22(i)-P2/2);
            Y(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            % Y(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            by(i) = by(i)...
                - y(getRow(i,n)+1,getCol(i,n)-1)*(-g12(i)/4-g21(i)/4);
            % Y(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            by(i) = by(i)...
                - y(getRow(i,n)-1,getCol(i,n)+1)*(-g12(i)/4-g21(i)/4);
            % Y(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            by(i) = by(i)...
                - y(getRow(i,n)-1,getCol(i,n)-1)*(g12(i)/4+g21(i)/4);
            
        % At eastern boundary (x = L)
        elseif etest && ~wtest && ~ntest && ~stest
            
            X(i,i) =  -2*g11(i)-2*g22(i);
            % X(i,i+1)=(g11(i)+P1/2) ;
            bx(i) = bx(i) - x(getRow(i,n),getCol(i,n)+1)*(g11(i)+P1/2);
            X(i,i-1)=(g11(i)-P1/2) ;
            X(i+n,i)=(g22(i)+P2/2) ;
            X(i-n,i)=(g22(i)-P2/2) ;
            % X(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            bx(i) = bx(i)...
                - x(getRow(i,n)+1,getCol(i,n)+1)*(g12(i)/4+g21(i)/4);
            X(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            % X(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            bx(i) = bx(i)...
                 - x(getRow(i,n)-1,getCol(i,n)+1)*(-g12(i)/4-g21(i)/4);
            X(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            
            Y(i,i) =  -2*g11(i)-2*g22(i);
            % Y(i,i+1)=(g11(i)+P1/2) ;
            by(i) = by(i) - y(getRow(i,n),getCol(i,n)+1)*(g11(i)+P1/2);
            Y(i,i-1)=(g11(i)-P1/2) ;
            Y(i+n,i)=(g22(i)+P2/2) ;
            Y(i-n,i)=(g22(i)-P2/2) ;
            % Y(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            by(i) = by(i)...
                 - y(getRow(i,n)+1,getCol(i,n)+1)*(g12(i)/4+g21(i)/4);
            Y(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            % Y(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            by(i) = by(i)...
                 - y(getRow(i,n)-1,getCol(i,n)+1)*(-g12(i)/4-g21(i)/4);
            Y(i-n,i-1)=(g12(i)/4+g21(i)/4); 

        % At western boundary
        elseif ~etest && wtest && ~ntest && ~stest
            
            X(i,i) =  -2*g11(i)-2*g22(i);
            X(i,i+1)=(g11(i)+P1/2) ;
            % X(i,i-1)=(g11(i)-P1/2) ;
            bx(i) = bx(i) - x(getRow(i,n),getCol(i,n)-1)*(g11(i)-P1/2);
            X(i+n,i)=(g22(i)+P2/2) ;
            X(i-n,i)=(g22(i)-P2/2) ;
            X(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            % X(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            bx(i) = bx(i)...
                 - x(getRow(i,n)+1,getCol(i,n)-1)*(-g12(i)/4-g21(i)/4);
            X(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            % X(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            bx(i) = bx(i)...
                 - x(getRow(i,n)-1,getCol(i,n)-1)*(g12(i)/4+g21(i)/4);
            
            Y(i,i) =  -2*g11(i)-2*g22(i);
            Y(i,i+1)=(g11(i)+P1/2) ;
            % Y(i,i-1)=(g11(i)-P1/2) ;
            by(i) = by(i) - y(getRow(i,n),getCol(i,n)-1)*(g11(i)-P1/2);
            Y(i+n,i)=(g22(i)+P2/2) ;
            Y(i-n,i)=(g22(i)-P2/2) ;
            Y(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            % Y(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            by(i) = by(i)...
                 - y(getRow(i,n)+1,getCol(i,n)-1)*(-g12(i)/4-g21(i)/4);
            Y(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            % Y(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            by(i) = by(i)...
                 - y(getRow(i,n)-1,getCol(i,n)-1)*(g12(i)/4+g21(i)/4);

        % At northern boundary (y = h)
        elseif ~etest && ~wtest && ntest && ~stest  

            X(i,i) =  -2*g11(i)-2*g22(i);
            X(i,i+1)=(g11(i)+P1/2) ;
            X(i,i-1)=(g11(i)-P1/2) ;
            % X(i+n,i)=(g22(i)+P2/2) ;
            bx(i) = bx(i) - x(getRow(i,n)+1,getCol(i,n))*(g22(i)+P2/2);
            X(i-n,i)=(g22(i)-P2/2) ;
            % X(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            bx(i) = bx(i)...
                 - x(getRow(i,n)+1,getCol(i,n)+1)*(g12(i)/4+g21(i)/4) ;
            % X(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            bx(i) = bx(i)...
                 - x(getRow(i,n)+1,getCol(i,n)-1)*(-g12(i)/4-g21(i)/4) ;
            X(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            X(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            
            Y(i,i) =  -2*g11(i)-2*g22(i);
            Y(i,i+1)=(g11(i)+P1/2) ;
            Y(i,i-1)=(g11(i)-P1/2) ;
            % Y(i+n,i)=(g22(i)+P2/2) ;
            by(i) = by(i) - y(getRow(i,n)+1,getCol(i,n))*(g22(i)+P2/2);
            Y(i-n,i)=(g22(i)-P2/2) ;
            % Y(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            by(i) = by(i)...
                 - y(getRow(i,n)+1,getCol(i,n)+1)*(g12(i)/4+g21(i)/4) ;
            % Y(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            by(i) = by(i)...
                 - y(getRow(i,n)+1,getCol(i,n)-1)*(-g12(i)/4-g21(i)/4) ;
            Y(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            Y(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            
            
        % At southern boundary (y = 0) 
        elseif ~etest && ~wtest && ~ntest && stest   

            X(i,i) = -2*g11(i)-2*g22(i);
            X(i,i+1)=(g11(i)+P1/2) ;
            X(i,i-1)=(g11(i)-P1/2) ;
            X(i+n,i)=(g22(i)+P2/2) ;
            % X(i-n,i)=(g22(i)-P2/2) ;
            bx(i) = bx(i) - x(getRow(i,n)-1,getCol(i,n))*(g22(i)-P2/2);
            X(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            X(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            % X(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            bx(i) = bx(i)...
                 - x(getRow(i,n)-1,getCol(i,n)+1)*(-g12(i)/4-g21(i)/4);
            % X(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            bx(i) = bx(i)...
                 - x(getRow(i,n)-1,getCol(i,n)-1)*(g12(i)/4+g21(i)/4);
            
            Y(i,i) = -2*g11(i)-2*g22(i);
            Y(i,i+1)=(g11(i)+P1/2) ;
            Y(i,i-1)=(g11(i)-P1/2) ;
            Y(i+n,i)=(g22(i)+P2/2) ;
            % Y(i-n,i)=(g22(i)-P2/2) ;
            by(i) = by(i) - y(getRow(i,n)-1,getCol(i,n))*(g22(i)-P2/2);
            Y(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            Y(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            % Y(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            by(i) = by(i)...
                 - y(getRow(i,n)-1,getCol(i,n)+1)*(-g12(i)/4-g21(i)/4);
            % Y(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            by(i) = by(i)...
                 - y(getRow(i,n)-1,getCol(i,n)-1)*(g12(i)/4+g21(i)/4);
            
        %Not at any boundary
        else 

            X(i,i) =  -2*g11(i)-2*g22(i);
            X(i,i+1)=(g11(i)+P1/2) ;
            X(i,i-1)=(g11(i)-P1/2) ;
            X(i+n,i)=(g22(i)+P2/2) ;
            X(i-n,i)=(g22(i)-P2/2) ;
            X(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            X(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            X(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            X(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            
            Y(i,i) =  -2*g11(i)-2*g22(i);
            Y(i,i+1)=(g11(i)+P1/2) ;
            Y(i,i-1)=(g11(i)-P1/2) ;
            Y(i+n,i)=(g22(i)+P2/2) ;
            Y(i-n,i)=(g22(i)-P2/2) ;
            Y(i+n,i+1)=(g12(i)/4+g21(i)/4) ;
            Y(i+n,i-1)=(-g12(i)/4-g21(i)/4) ;
            Y(i-n,i+1)=(-g12(i)/4-g21(i)/4) ;
            Y(i-n,i-1)=(g12(i)/4+g21(i)/4); 
            
        end % if

        etest = false;
        wtest = false;
        ntest = false;
        stest = false;
   
    end %for

    xx = X\bx';                                          % Matrix inversion
    yy = Y\by';                                          % Matrix inversion
    
    x_mat = global2matrix(xx, n, m);
    y_mat = global2matrix(yy, n, m);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Check convergence
    cx = max(max(abs(x_mat-x(2:M-1,2:N-1))));
    cy = max(max(abs(y_mat-y(2:M-1,2:N-1))));
    
    cx_lim = 10^-3;
    cy_lim = 10^-3;
    
    if (cx < cx_lim && cy < cy_lim ) || it == maxits
        conv = 1; % Stop
    else

        it = it + 1;
    end %if
    
    % Under-relaxation:
    x(2:M-1,2:N-1) = (1-alpha)*x(2:M-1,2:N-1) +  alpha * x_mat;
    y(2:M-1,2:N-1) = (1-alpha)*y(2:M-1,2:N-1) + alpha * y_mat;
    
end % while
figure 
plot(x,y,'k',x',y','k')
xlim([xA,xD])
ylim([yD,yC])
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
saveas(gcf,'elliptic.png')
